package dsnserver;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dsnserver.CmdParserLogging.dsn;

/**********************************************
 * It is for parsing the ID messages which are
 * sent back due to a ping request. 
 * It is used to update the dsninfo table which 
 * shows all DSNnode which was seen since the 
 * DSNServer has started and to detect reoobts 
 * of the nodes
 *  
 * @author kaltt
 *
 **********************************************/
public class CmdParserDSNID implements CmdParser{

	public CmdParserDSNID() {
		super();
		// TODO Auto-generated constructor stub
	}
	
	
	/**********************************************
	 * It is responsible for the lines which start
	 * with ":DP". It parses the DSNID an write it
	 * to the DB if this DSNID doesn't already exist,
	 * otherwise it will update the existing row with 
	 * the new time. 
	 * Additionaly it detect reboots of the nodes. 
	 * If a reboot was detected it will write a
	 * log message to the log table.
	 *  
	 * @author kaltt
	 *
	 **********************************************/
	public void parseLine(String line){
		line = line.replaceAll("'","''");
		String[] token = line.split(" ");
		Pattern p = Pattern.compile(":DP ([A-Fa-f0-9]{2}:){5}[A-Fa-f0-9]{2} [0-9]+");
		Matcher m = p.matcher(line.trim());
		if (m.matches() && token.length == 3){			
//		if (token.length == 3){
			String DSNID = "";
			String rebootNr = token[2].trim();
			if (token[1].contains(":") && (token[1].length()==5 || token[1].length()==17)){
				if (token[1].length()<=5) {DSNID = CmdParserGetAddr.DSNpreAddr + token[1];}else{DSNID = token[1];}
				String BootCount = "";
				SQLDBcommunication.lock();
				ResultSet result_sql = SQLDBcommunication.getResultsFromDB("" +
						"SELECT `BootCount` FROM `dsninfo` " +
						"WHERE `DSNID` = '"+DSNID+"' LIMIT 0 , 1");
				try {
					if (result_sql.next()){ BootCount = result_sql.getString("BootCount");}
				} catch (SQLException e) {}
				SQLDBcommunication.unlock();
				//BootCount is the value out of the DSNinfo talbe
				//rebootNr is the value which comes from the answer of last DSN Ping
				if (!BootCount.equals("")){
					/**********************************************
					 * if the the DSN node did not reboot the value 
					 * out of the DB and from the last DSN ping are 
					 * the same. 
					 * the BootCount entry in the DSNinfo table is 
					 * per default 0. If there was no dsn ping 
					 * before BootCount is equal 0. 
					 * In this cases there will be just an update 
					 * of the DSNlastSeen entry in the DB. 
					 **********************************************/
					
					if (CmdParserLogging.DSN.containsKey(DSNID)){
						CmdParserLogging.DSN.get(DSNID).BootCount = Integer.parseInt(rebootNr.trim());
					}
					if (BootCount.equals(rebootNr) || BootCount.equals("0")){
						SQLDBcommunication.lock();
						SQLDBcommunication.sendQueryToDB("" +
								"UPDATE `dsninfo` SET " +
								"`BootCount` = '"+ rebootNr +"', " +
								"`DSNlastSeen` = NOW( ) " +
								"WHERE `DSNID` = '" + DSNID + "';");
						SQLDBcommunication.unlock();
					}
					/**********************************************
					 * Otherwise there must be occured an reboot. 
					 * The DSNinfo table will be updated with the 
					 * new CootCount number and the DSNlastSeen entry
					 * will be updated as well.
					 * DSN.log & DSN.received & DSN.request will be
					 * cleared. Additonaly there will be inserted the
					 * value 1, to garanty that the all new logs, started 
					 * with 1, will be requested. 
					 * received = 1: if 1,2 and thre will lost and 4 is
					 * the first that will arrive. updateRequestLog() will 
					 * notice that 2 and 3 has to be requested and 1 is 
					 * already in request list.
					 * Furthermore it will be generated an log entry 
					 * to show that a node was rebooted.   
					 **********************************************/
					else {
						SQLDBcommunication.lock();
						SQLDBcommunication.sendQueryToDB("" +
								"UPDATE `dsninfo` SET " +
								"`BootCount` = '"+ rebootNr +"', " +
								"`DSNlastSeen` = NOW( ) " +
								"WHERE `DSNID` = '" + DSNID + "';");
						SQLDBcommunication.unlock();
						if (CmdParserLogging.DSN.containsKey(DSNID)){
							if (!CmdParserLogging.DSN.get(DSNID).log.isEmpty()){CmdParserLogging.DSN.get(DSNID).log.clear();}
							if (!CmdParserLogging.DSN.get(DSNID).received.isEmpty()) {CmdParserLogging.DSN.get(DSNID).received.clear();}
							if (!CmdParserLogging.DSN.get(DSNID).request.isEmpty()) {CmdParserLogging.DSN.get(DSNID).request.clear();}
						}
						if (CmdParserLogging.DSN.containsKey(DSNID)){
							//CmdParserLogging.DSN.get(DSNID).request.add(1);
							//CmdParserLogging.DSN.get(DSNID).received.add(1);
						}
						SQLDBcommunication.lock();
						SQLDBcommunication.sendQueryToDB("" +
								"INSERT INTO `dsnlog` " +
								"( `DSNID` , `MsgID` , `BootCount`, `DSNtime` , " +
								"`time` , `Class` , `Level` , `DBtime` , `LogText` ) " +
								"VALUES ( '"+DSNID+"', 0, "+rebootNr+", '0', " +
								"CURRENT_TIMESTAMP, 0, 0, CURRENT_TIMESTAMP , 'reboot of node "+DSNID+"');");
						SQLDBcommunication.unlock();
					}
				/**********************************************
				 * If there was no entry in the DSNinfo table. 
				 **********************************************/					
				}else{
					SQLDBcommunication.lock();
					SQLDBcommunication.sendQueryToDB("" +
							"INSERT INTO `dsninfo` " +
							"(`DSNID` , `BootCount`, `DSNlastSeen` )" +
							"VALUES ('" + DSNID + "', '" + rebootNr + "', NOW( ));");
					SQLDBcommunication.unlock();
					if (!CmdParserLogging.DSN.containsKey(DSNID)){
						dsn input = new dsn();
						CmdParserLogging.DSN.put(DSNID,input);
						CmdParserLogging.DSN.get(DSNID).BootCount = Integer.parseInt(rebootNr.trim());
					}
				}
			}
		}
	}
}

