package dsnserver;
import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.Semaphore;

import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;


/**********************************************
 * It is used for the communication with the 
 * BTnode over the serial port.   
 * 
 * @author kaltt
 *
 *********************************************/
public class RXTXComm implements Communication {

	public CommPortIdentifier portId;
	private SerialPort port;
	private String portName = GetProperty.get("portName");
	private int baudrate = Integer.parseInt(GetProperty.get("baudrate"));
	private InputStream is;
        private BufferedReader in;
	private OutputStream os;
	public PrintStream out;
	protected static String defaultLogFile = GetProperty.get("logFilename","msglog.txt");
	
	private Semaphore sem = new Semaphore(1, true);
	private static Semaphore sem_file = new Semaphore(1, true);
	 
	protected void finalize() {
                close();
	}
		
	/**********************************************
	 * To open a connection to a special Socket.
	 * Create a InputStream (is) 
	 * and a OutputStream (os)
	 * For example with the COM3 Seriel-Connection
	 * portName = "COM3"
	 * 
	 * @param 
	 * String Name
	 **********************************************/
	public void open(String Name) {
		try {
			portId = CommPortIdentifier.getPortIdentifier(portName);
			port = (SerialPort) portId.open(Name, 5000);
			port.setSerialPortParams(baudrate, SerialPort.DATABITS_8,
					SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
			is = port.getInputStream();
			in = new BufferedReader(new InputStreamReader(is));
			os = port.getOutputStream();
			out = new PrintStream(os);
			
			System.out.println("Sucessfully opened COM port " + portName);
			
		} catch (Exception e) {
			System.out.println("Unable to open COM port " + portName);
			showPorts();
			System.err.println(e);
			e.printStackTrace();
			System.exit(0);
		}
	}
	
	/**********************************************
	 * To close a connection.
	 **********************************************/	
	public void close() {
		try {
			//System.out.print("RXTXComm: close()... ");
                        in.close();
			out.close();
			in.close();
			os.close();
			is.close();
			port.close();
			//System.out.println("done");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}	
	
	/**********************************************
	* to show Ports
	*  <br>
	* Ports found: <br>
	*  - Name: COM1 <br>
	*  - String: gnu.io.CommPortIdentifier@56a499 <br>
	*  - PortType: 1 <br>
	*  - Hash: 5678233 <br>
	*  - owner: null <br>
	*  - Name: COM2  <br>
	*  - ...
	**********************************************/
	protected void showPorts() {
			CommPortIdentifier portId;
			Enumeration portenum = CommPortIdentifier.getPortIdentifiers();
			System.out.print("Ports found:\n");
			while (portenum.hasMoreElements()) {
				portId = (CommPortIdentifier) portenum.nextElement();
				System.out.print("  - Name: " + portId.getName() + "\n");
				System.out.print("  - String: " + portId.toString() + "\n");
				System.out.print("  - PortType: " + portId.getPortType() + "\n");
				System.out.print("  - Hash: " + portId.hashCode() + "\n");
				System.out.print("  - owner: " + portId.getCurrentOwner() + "\n");
			}
	}

	public String readLine() {
		String line = null;
		do {
			try {
				line = in.readLine() + "\n";
			} catch (IOException e) { }
		} 
		while (line == null || line.equals(""));
		
		try {
			if (GetProperty.get("logFile").equals("yes")) { 
				writeLog("< " + line);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return line;
	}
	
	/**********************************************
	* To write a line on the com-port
	* It has to be safed with a semaphre. For this 
	* you can use before the writeLine command the 
	* command lock() and after unlock().
	* 
	* @param
	* String command the command wich is written 
	* to the port to the BTnode. 
	**********************************************/		
	public void writeLine(String command) {
		try {
			if (GetProperty.get("logFile").equals("yes")) {
				writeLog("> " + command);
			}
			command = command + "\n";
			byte[] command_string = command.getBytes();
			out.write(command_string);
			//System.err.println(command);			 
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static void writeLog(String s) throws IOException {
		if (GetProperty.get("logFile").equals("yes")) {
			writeLog(defaultLogFile, s);
		}
	}

	public static void writeLog(String f, String s) throws IOException {
		//TimeZone tz = TimeZone.getTimeZone("EST"); // or PST, MID, etc ...
		s=s.trim();	
		if (s.length()>1){
			try {
				sem_file.acquire();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			Date now = new Date();
			DateFormat df = new SimpleDateFormat ("yyyy-MM-dd HH:mm:ss:SSS "); 
			//df.setTimeZone(tz);
			String currentTime = df.format(now);
			try {
				FileWriter aWriter = new FileWriter(f, true);
				aWriter.write(currentTime + " " + s + System.getProperty("line.separator"));
				aWriter.flush();
				aWriter.close();
			}
			catch(IOException e) {
				System.err.println("File error "+e.toString());
				System.out.println(currentTime + " " + s);
			}
			sem_file.release();
		}
	}
  
	public void deleteLog() {
		File f = new File(defaultLogFile);
		// Make sure the file or directory exists and isn't write protected
//		if (!f.exists())
//			throw new IllegalArgumentException(
//				"Delete: no such file or directory: " + defaultLogFile);
//		if (!f.canWrite())
//			throw new IllegalArgumentException(
//				"Delete: write protected: " + defaultLogFile);
//		// If it is a directory, make sure it is empty
//		if (f.isDirectory()) {
//			String[] files = f.list();
//			if (files.length > 0)
//				throw new IllegalArgumentException(
//					"Delete: directory not empty: " + defaultLogFile);
//		}
		// Attempt to delete it
		if (f.exists()) {
			boolean success = f.delete();
			if (!success)
				throw new IllegalArgumentException("Delete: deletion failed");
		}
	}
	
	public void lock() {
		try {
			sem.acquire();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void unlock(){
		sem.release();
	}
	
	public void filelock() {
		try {
			sem_file.acquire();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void fileunlock(){
		sem_file.release();
	}
}
