package dsnserver;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.Semaphore;

/**********************************************
 * It is used for the communication with the 
 * MySQL Database.   
 * 
 * @author kaltt
 *
 *********************************************/
public class SQLDBcommunication {

	static String mySqlDriver = GetProperty.get("mySqlDriver");
	static String mySqlUrl = 	GetProperty.get("mySqlUrl");
	static String mySqlUser = GetProperty.get("mySqlUser");
	static String mySqlPasswd = GetProperty.get("mySqlPasswd");

	private static Semaphore sem = new Semaphore(1, true);

	private static Connection con;
	private static Statement stmt;
	
	/**********************************************
	 * To send a query to the DB.
	 * 
	 * @param 
	 * String query
	 * 
	 * @return
	 * ResultSet rs
	 **********************************************/
	public static ResultSet getResultsFromDB(String query) {
		//Statement stmt;
		ResultSet rs = null;
		try {
			//Get a Statement object
			if (con.isClosed()){
				connectToDB();
//System.out.println("connect to DB!!");				
			}
//			stmt = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
//					ResultSet.CONCUR_READ_ONLY);
			//Query the database, storing the result
			// in an object of type ResultSet
			rs = stmt.executeQuery(query);
			//con.close();
		}catch (SQLException E) {
			 Date now = new Date();
		     DateFormat df = new SimpleDateFormat ("yyyy-MM-dd HH:mm:ss "); 
		     String currentTime = df.format(now); 
			 
		     System.out.println("SQLException: " + E.getMessage());
			 System.out.println("Time:         " + currentTime);
			 System.out.println("SQLStatement: " + query);
		     System.out.println("SQLState:     " + E.getSQLState());
		     System.out.println("VendorError:  " + E.getErrorCode());
		     try {
		    	 //to give a empty ResultSet back!
				rs = stmt.executeQuery("SELECT * FROM `dsninfo` LIMIT 0");
			} catch (SQLException e) {
				 System.out.println("SQLException: " + E.getMessage());
			     System.out.println("SQLState:     " + E.getSQLState());
			     System.out.println("VendorError:  " + E.getErrorCode());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return rs;
	}

	/**********************************************
	 * To send a query to the DB.
	 * For the Statemant DROP TABLE, CREATE TABLE 
	 * and INSERT give back the number of affected 
	 * rows. 
	 *  
	 * @param 
	 * String query
	 * 
	 * @return
	 * int count
	 **********************************************/	
	public static int sendQueryToDB(String query) {
		//Statement stmt;
		int count = 0;
		try {
			//Get a Statement object
			if (con.isClosed()){
				connectToDB();
//System.out.println("connect to DB!!");				
			}
//			stmt = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
//					ResultSet.CONCUR_READ_ONLY);
			// For the Statemant DROP TABLE, CREATE TABLE, and INSERT
			// give back the number of affected rows 
			count = stmt.executeUpdate(query);
			//con.close();
		}catch (SQLException E) {
			if (E.getErrorCode()!=1062){
				 System.out.println("SQLStatement: " + query);
				 System.out.println("SQLException: " + E.getMessage());
			     System.out.println("SQLState:     " + E.getSQLState());
			     System.out.println("VendorError:  " + E.getErrorCode());
			}
//else{
//     System.out.println("Duplicated Entry!!");
//	 System.out.println("SQLStatement: " + query);
//}
		} catch (Exception e) {
			e.printStackTrace();
		} 
		return count;
	}
	
	
	/**********************************************
	 * To open a connnection to a MySQL DB
	 * 
	 * @throws ClassNotFoundException
	 * @throws SQLException
	 **********************************************/
	public static void connectToDB() throws ClassNotFoundException,
			SQLException {
		//Register the JDBC driver for MySQL.
		Class.forName(mySqlDriver);
		
		//Define URL of database server for
		// database named mysql on the localhost
		// with the default port number 3306.		                                                 
		String url = mySqlUrl+"?zeroDateTimeBehavior=convertToNull&amp;autoReconnect=true&amp;jdbcCompliantTruncation=false ";


		//Get a connection to the database for a
		// user named root with a blank password.
		// This user is the default administrator
		// having full privileges to do anything.
		con = DriverManager.getConnection(url, mySqlUser, mySqlPasswd);
		//if (stmt!=null){stmt.close();}
		stmt = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_READ_ONLY,ResultSet.HOLD_CURSORS_OVER_COMMIT);
	}

	/**********************************************
	 * To close the connection to the DB
	 **********************************************/	
	public static void closeConnectionToDB() throws ClassNotFoundException,
			SQLException {
		con.close();
	}
	
	public static void lock() {
		try {
			sem.acquire();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void unlock(){
		sem.release();
	}

	
	public static void makeTable(){
		SQLDBcommunication.lock();
		sendQueryToDB(
		"CREATE TABLE IF NOT EXISTS dsninfo ( " +
		"ID int(11) unsigned NOT NULL auto_increment, " +
		"DSNID varchar(17) collate latin1_general_ci default '', " +
		"BootCount smallint(5) unsigned default '0', " +
		"DSNVer varchar(50) collate latin1_general_ci default '', " +
		"DSNLocation varchar(150) collate latin1_general_ci default '', " +
		"DSNProgInfo varchar(150) collate latin1_general_ci default '', " +
		"DSNlastSeen timestamp NOT NULL default CURRENT_TIMESTAMP, " +
		"`NodeInfo` varchar(255) collate latin1_general_ci NOT NULL default '', " +
		"PRIMARY KEY  (ID) " +
		") ENGINE=MyISAM DEFAULT CHARSET=latin1 COLLATE=latin1_general_ci;" +
		"");
		SQLDBcommunication.unlock();
		
		SQLDBcommunication.lock();
		sendQueryToDB(
		"CREATE TABLE IF NOT EXISTS dsnlog (" +
		"  ID int(10) unsigned NOT NULL auto_increment," +
		"  DSNID char(17) NOT NULL default ''," +
		"  BootCount smallint(5) unsigned NOT NULL," +
		"  MsgID smallint(5) unsigned NOT NULL," +
		"  DSNtime int(10) unsigned NOT NULL," +
		"  `time` datetime NOT NULL default '1970-01-01 00:00:00'," +
		"  Class tinyint(3) unsigned NOT NULL default '0'," +
		"  `Level` tinyint(3) unsigned NOT NULL default '0',  " +
		"  DBtime timestamp NOT NULL default CURRENT_TIMESTAMP," +
		"  LogText text NOT NULL," +
		"  PRIMARY KEY  (ID)," +
		"  UNIQUE KEY DSNID (DSNID,BootCount,MsgID,DSNtime)" +
		") ENGINE=MyISAM DEFAULT CHARSET=latin1 PACK_KEYS=0;" 
		);
		SQLDBcommunication.unlock();
		
		SQLDBcommunication.lock();
		sendQueryToDB(
		"CREATE TABLE IF NOT EXISTS `dsnpwd` (" +
		"`ID` int(11) NOT NULL auto_increment," +
		" `name` varchar(30) collate latin1_general_ci NOT NULL, " +
		"`pwd` varchar(32) collate latin1_general_ci NOT NULL," +
		" `function` varchar(60) collate latin1_general_ci NOT NULL," +
		" PRIMARY KEY  (`ID`)" +
		") ENGINE=MyISAM DEFAULT CHARSET=latin1 COLLATE=latin1_general_ci AUTO_INCREMENT=2 ;" 
		);
		SQLDBcommunication.unlock();
		
		SQLDBcommunication.lock();
		sendQueryToDB(
		"CREATE TABLE IF NOT EXISTS `dsnconinfo` (" +
		" `ID` int(11) NOT NULL auto_increment," +
		" `time` timestamp NOT NULL default CURRENT_TIMESTAMP," +
		" `SourceAddr` varchar(17) collate latin1_general_ci NOT NULL, " +
		" `NeighborAddr` varchar(17) collate latin1_general_ci NOT NULL," +
		" `State` tinyint(4) NOT NULL," +
		" `rssi` tinyint(4) NOT NULL," +
		" PRIMARY KEY  (`ID`));"
		); 
		SQLDBcommunication.unlock();
		
		SQLDBcommunication.lock();
		sendQueryToDB(
		"CREATE TABLE IF NOT EXISTS `dsnrpcs` (" +
		" `ID` int(11) NOT NULL auto_increment," +
		" `time` timestamp NOT NULL default CURRENT_TIMESTAMP," +
		" `DSNID` varchar(17) collate latin1_general_ci NOT NULL, " +
		" `msg` text collate latin1_general_ci NOT NULL," +
		" PRIMARY KEY  (`ID`));"
		);
		SQLDBcommunication.unlock();

	}
}
