package dsnserver;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;
import java.util.Arrays;

import org.apache.xmlrpc.AuthenticatedXmlRpcHandler;
import org.apache.xmlrpc.WebServer;

/**********************************************
 * A function which is used as a XmlRpc function.
 * It is used for sending out a DSN command. 
 *
 * @author kaltt
 * 
 **********************************************/
public class XmlRpcDSNCommand implements AuthenticatedXmlRpcHandler{

	Communication myComm;
	WebServer server;
	
	private final String[] realRPCcommands = {"xbank"};
	
	public XmlRpcDSNCommand(Communication comm,WebServer server){
		this.server = server;
		myComm = comm;
	}

	/**********************************************
	* A function which is used as a XmlRpc function.
	* It is used for sending out a DSN command.
	*  
	* @param String DSNID<br>
	* String command (the command for the DSN node)
	* 
	* @return return "";
	**********************************************/
	public String dsnCommand(String DSNID, String command) {
		
		if (!(DSNID.equals("root") || DSNID.equals(CmdParserGetAddr.DSNnodeAddr))){
			// destination is not root (gui) node
			
			if (DSNID.equals("all")) {
				DSNID = "0";
			}
			
			String rpctype;
                        // use rpc only with 'dsn ping' (for the first time)
			if (Arrays.asList(realRPCcommands).contains(command.split(" ")[0].trim()) || 
                                (command.split(" ")[0].trim().equals("dsn") && command.split(" ")[1].trim().equals("ping"))) {
				// command is a real rpc command
                                rpctype = "rpc";
			}
			else{
				rpctype = "rpcs";
			}
			
			command = rpctype + " " + DSNID + " " + command;
		}

		myComm.lock();
                myComm.writeLine(command);
		myComm.unlock();
		
		return "";
	}

	public Object execute(String method, Vector v, String user,  String password) {
		if(checkPassword(user, password)){
			return execute(method, v);
		}
		else{
			try {
				RXTXComm.writeLog("Error in DSNCommand: Unauthorized User (user:" + user + ")");
			} catch (IOException e1) {}
			return null;
		}
	}

	protected Object execute(String method, Vector v){
		String result = null;
		if(method.endsWith(".dsnCommand")){
			try {
				result = dsnCommand((String)v.get(0),(String)v.get(1));
			} catch (NumberFormatException e) {
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return result;
	}
	
	public boolean checkPassword(String user, String password){
		boolean pwd = false;
		SQLDBcommunication.lock();
		ResultSet result = SQLDBcommunication.getResultsFromDB("" +
				"SELECT *  FROM `dsnpwd` " +
				"WHERE `name` = '" + user + "' " +
				"AND `pwd` = md5('" + password + "') " +
				"AND (`function` =  'dsnCommand' OR `function` =  'all' )");
		try {
			if (result.next()){pwd = true;}
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		SQLDBcommunication.unlock();
		if (pwd) return true;
		else return false;
	}


}
