package dsnserver;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

import org.apache.xmlrpc.AuthenticatedXmlRpcHandler;
import org.apache.xmlrpc.WebServer;

/**********************************************
 * A function which is used as a XmlRpc function.
 * It is used for defining the log class which 
 * is sent directly to the GUInode. 
 *
 * @author kaltt
 * 
 **********************************************/
public class XmlRpcDSNVerbosity implements AuthenticatedXmlRpcHandler{

	Communication myComm;
	WebServer server;

	public XmlRpcDSNVerbosity(Communication comm,WebServer server){
		this.server = server;
		myComm = comm;
	}

	/**********************************************
	* It is used for defining the log class which 
	* is send directly to the GUInode. 
	*  
	* @param String DSNID<br>
	* int Class (which class should be sent back directly)<br>
	* int Levelmask (which level of the class should be sent back directly ((dez x01-Error x02-Warning x03-Info x04-Debug x05-Reply) (e.g.: set all level = 31, set none of them = 0, set Error & Warning = 3))
	* 
	* @return return "";
	**********************************************/
	public String dsnVerbosity(String DSNID, int Class, int Levelmask) {
		
       if (DSNID.equals("all")){DSNID = "00:00:00:00:00:00";}
       		
		String Level = "";
        switch (Levelmask) {
	        case 0:   Level = "00"; break;
	        case 1:   Level = "01"; break; 
	        case 2:   Level = "02"; break; 
	        case 3:   Level = "03"; break; 
	        case 4:   Level = "04"; break; 
	        case 5:   Level = "05"; break; 
	        case 6:   Level = "06"; break;
	        case 7:   Level = "07"; break;
	        case 8:   Level = "08"; break;
	        case 9:   Level = "09"; break;
	        case 10:  Level = "0A"; break;
	        case 11:  Level = "0B"; break;
	        case 12:  Level = "0C"; break;
	        case 13:  Level = "0D"; break;
	        case 14:  Level = "0E"; break;
	        case 15:  Level = "0F"; break;
	        case 16:  Level = "10"; break;
	        case 17:  Level = "11"; break;
	        case 18:  Level = "12"; break;
	        case 19:  Level = "13"; break;
	        case 20:  Level = "14"; break;
	        case 21:  Level = "15"; break;
	        case 22:  Level = "16"; break;
	        case 23:  Level = "17"; break;
	        case 24:  Level = "18"; break;
	        case 25:  Level = "19"; break;
	        case 26:  Level = "1A"; break;
	        case 27:  Level = "1B"; break;
	        case 28:  Level = "1C"; break;
	        case 29:  Level = "1D"; break;
	        case 30:  Level = "1E"; break;
	        case 31:  Level = "1F"; break;
	        default:  Level = "FF"; break;
        }
 		
 		myComm.lock();
        myComm.writeLine("rpcs " + DSNID + " slog vtarget " + CmdParserGetAddr.DSNnodeAddr);
        myComm.writeLine("rpcs " + DSNID + " slog verbosity " + Class + " " + Level);
		myComm.unlock();
		
		return "";
	}
	
	public Object execute(String method, Vector v, String user,  String password) {
		if(checkPassword(user, password)){
			return execute(method, v);
		}
		else{
			try {
				RXTXComm.writeLog("Error in DSNCommand: Unauthorized User (user:" + user + ")");
			} catch (IOException e1) {}  
			return null;			
		}
	}

	protected Object execute(String method, Vector v){
		String result = null;
		if(method.endsWith(".dsnVerbosity")){
			try {
				result = dsnVerbosity((String)v.get(0),(Integer)v.get(1),(Integer)v.get(2));
			} catch (NumberFormatException e) {
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return result;
	}
	
	public boolean checkPassword(String user, String password){
		boolean pwd = false;
		SQLDBcommunication.lock();
		ResultSet result = SQLDBcommunication.getResultsFromDB("" +
				"SELECT *  FROM `dsnpwd` " +
				"WHERE `name` = '" + user + "' " +
				"AND `pwd` = md5('" + password + "') " +
				"AND (`function` =  'dsnVerbosity' OR `function` =  'all' )");
		try {
			if (result.next()){pwd = true;}
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		SQLDBcommunication.unlock();
		if (pwd) return true;
		else return false;
	}

}
